@@ -1,6 +1,6 @@
 /************************************************************************
  *
- * $Id: geos_c.cpp 2742 2009-11-26 18:29:56Z warmerdam $
+ * $Id: geos_c.cpp 2975 2010-04-17 21:55:53Z strk $
  *
  * C-Wrapper for GEOS library
  *
@@ -85,10 +85,23 @@ GEOSContextHandle_t handle = NULL;
 
 extern "C" {
 
+GEOSMessageHandler
+GEOSContext_setNoticeHandler_r(GEOSContextHandle_t extHandle, GEOSMessageHandler nf);
+GEOSMessageHandler
+GEOSContext_setErrorHandler_r(GEOSContextHandle_t extHandle, GEOSMessageHandler nf);
+
 void
 initGEOS (GEOSMessageHandler nf, GEOSMessageHandler ef)
 {
-    handle = initGEOS_r( nf, ef );
+    if ( ! handle )
+    {
+        handle = initGEOS_r( nf, ef );
+    }
+    else
+    {
+        GEOSContext_setNoticeHandler_r(handle, nf);
+        GEOSContext_setErrorHandler_r(handle, ef);
+    }
 }
 
 void
