@@ -1,9 +1,10 @@
 /************************************************************************
  *
- * $Id: geos_ts_c.cpp 2757 2009-12-01 15:39:41Z mloskot $
+ * $Id: geos_ts_c.cpp 2975 2010-04-17 21:55:53Z strk $
  *
  * C-Wrapper for GEOS library
  *
+ * Copyright (C) 2010 Sandro Santilli <strk@keybit.net>
  * Copyright (C) 2005-2006 Refractions Research Inc.
  *
  * This is free software; you can redistribute and/or modify it under
@@ -182,6 +183,40 @@ initGEOS_r(GEOSMessageHandler nf, GEOSMessageHandler ef)
     return static_cast<GEOSContextHandle_t>(extHandle);
 }
 
+GEOSMessageHandler
+GEOSContext_setNoticeHandler_r(GEOSContextHandle_t extHandle, GEOSMessageHandler nf)
+{
+    GEOSMessageHandler f;
+    GEOSContextHandleInternal_t *handle = 0;
+    handle = reinterpret_cast<GEOSContextHandleInternal_t*>(extHandle);
+    if ( 0 == handle->initialized )
+    {
+        return NULL;
+    }
+
+    f = handle->NOTICE_MESSAGE;
+    handle->NOTICE_MESSAGE = nf;
+
+    return f;
+}
+
+GEOSMessageHandler
+GEOSContext_setErrorHandler_r(GEOSContextHandle_t extHandle, GEOSMessageHandler nf)
+{
+    GEOSMessageHandler f;
+    GEOSContextHandleInternal_t *handle = 0;
+    handle = reinterpret_cast<GEOSContextHandleInternal_t*>(extHandle);
+    if ( 0 == handle->initialized )
+    {
+        return NULL;
+    }
+
+    f = handle->ERROR_MESSAGE;
+    handle->ERROR_MESSAGE = nf;
+
+    return f;
+}
+
 void
 finishGEOS_r(GEOSContextHandle_t extHandle)
 {
@@ -598,7 +633,10 @@ GEOSisValidReason_r(GEOSContextHandle_t extHandle, const Geometry *g1)
             TopologyValidationError *err = ivo.getValidationError();
             if (0 != err)
             {
-                const std::string errloc = err->getCoordinate().toString();
+		std::ostringstream ss;
+		ss.precision(15);
+		ss << err->getCoordinate();
+                const std::string errloc = ss.str();
                 std::string errmsg(err->getMessage());
                 errmsg += "[" + errloc + "]";
                 result = gstrdup(errmsg);
