@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: RobustDeterminant.cpp 2148 2008-08-07 16:59:14Z pramsey $
+ * $Id: RobustDeterminant.cpp 2764 2009-12-03 19:03:04Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -15,28 +15,18 @@
  *
  **********************************************************************
  *
- * RobustDeterminant implements an algorithm to compute the
- * sign of a 2x2 determinant for double precision values robustly.
- * It is a direct translation of code developed by Olivier Devillers.
- * 
- *************************************************************************
- * The original code carries the following copyright notice:
- * Author : Olivier Devillers
- * Olivier.Devillers@sophia.inria.fr
- * http:/www.inria.fr:/prisme/personnel/devillers/anglais/determinant.html
- **************************************************************************
+ * Last port: algorithm/RobustDeterminant.java 1.15 (JTS-1.10)
  *
- **************************************************************************
- *              Copyright (c) 1995  by  INRIA Prisme Project
- *                  BP 93 06902 Sophia Antipolis Cedex, France.
- *                           All rights reserved
- **************************************************************************
- */
+ **********************************************************************/
 
 #include <geos/algorithm/RobustDeterminant.h>
 
 #include <cmath>
 
+#ifdef _MSC_VER
+#pragma warning(disable : 4127)
+#endif
+
 namespace geos {
 namespace algorithm { // geos.algorithm
 
