@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: CoordinateSequence.cpp 2091 2007-12-21 22:49:03Z mloskot $
+ * $Id: CoordinateSequence.cpp 2793 2009-12-04 15:07:37Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -25,6 +25,7 @@
 #include <algorithm>
 #include <vector>
 #include <cassert>
+#include <iterator>
 
 using namespace std;
 
@@ -141,6 +142,21 @@ CoordinateSequence::scroll(CoordinateSequence* cl,
 	cl->setPoints(v);
 }
 
+int
+CoordinateSequence::increasingDirection(const CoordinateSequence& pts)
+{
+	size_t ptsize = pts.size();
+	for (size_t i=0, n=ptsize/2; i<n; ++i)
+	{
+		size_t j = ptsize - 1 - i;
+		// skip equal points on both ends
+		int comp = pts[i].compareTo(pts[j]);
+		if (comp != 0) return comp;
+	}
+	// array must be a palindrome - defined to be in positive direction
+	return 1;
+}
+
 void
 CoordinateSequence::reverse(CoordinateSequence *cl)
 {
