@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: PointInRing.h 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: PointInRing.h 2556 2009-06-06 22:22:28Z strk $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -17,6 +17,7 @@
 #ifndef GEOS_ALGORITHM_POINTINRING_H
 #define GEOS_ALGORITHM_POINTINRING_H
 
+#include <geos/export.h>
 
 // Forward declarations
 namespace geos {
@@ -28,7 +29,7 @@ namespace geos {
 namespace geos {
 namespace algorithm { // geos::algorithm
 
-class PointInRing{
+class GEOS_DLL PointInRing{
 public:
 	virtual ~PointInRing(){};
 	virtual bool isInside(const geom::Coordinate& pt)=0;
