@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: StringTokenizer.h 1820 2006-09-06 16:54:23Z mloskot $
+ * $Id: StringTokenizer.h 2775 2009-12-03 19:38:12Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -12,17 +12,23 @@
  * by the Free Software Foundation. 
  * See the COPYING file for more information.
  *
+ **********************************************************************
+ *
+ * Last port: ORIGINAL WORK
+ *
  **********************************************************************/
 
 #ifndef GEOS_IO_STRINGTOKENIZER_H
 #define GEOS_IO_STRINGTOKENIZER_H
 
+#include <geos/export.h>
+
 #include <string>
 
 namespace geos {
 namespace io {
 
-class StringTokenizer {
+class GEOS_DLL StringTokenizer {
 public:
 	enum {
 		TT_EOF,
@@ -42,6 +48,10 @@ private:
 	std::string stok;
 	double ntok;
 	std::string::const_iterator iter;
+
+    // Declare type as noncopyable
+    StringTokenizer(const StringTokenizer& other);
+    StringTokenizer& operator=(const StringTokenizer& rhs);
 };
 
 } // namespace io
