@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: SingleInteriorIntersectionFinder.h 2137 2008-07-25 21:00:09Z mloskot $
+ * $Id: SingleInteriorIntersectionFinder.h 2778 2009-12-03 19:44:00Z mloskot $
  *
  * GEOS - Geometry Engine Open Source
  * http://geos.refractions.net
@@ -44,11 +44,6 @@ namespace noding { // geos.noding
 class SingleInteriorIntersectionFinder: public SegmentIntersector
 {
 
-private:
-	algorithm::LineIntersector& li;
-	geom::Coordinate interiorIntersection;
-	std::vector<geom::Coordinate> intSegments;
-
 public:
 
 	/** \brief
@@ -71,7 +66,7 @@ public:
 	 */
 	bool hasIntersection() const
 	{ 
-		return interiorIntersection != geom::Coordinate::getNull(); 
+		return !interiorIntersection.isNull(); 
 	}
   
 	/** \brief
@@ -110,9 +105,17 @@ public:
   
 	bool isDone() const
 	{ 
-		// TODO: add Coordinate::isNull()
-		return interiorIntersection != geom::Coordinate::getNull();
+		return !interiorIntersection.isNull();
 	}
+
+private:
+	algorithm::LineIntersector& li;
+	geom::Coordinate interiorIntersection;
+	std::vector<geom::Coordinate> intSegments;
+
+    // Declare type as noncopyable
+    SingleInteriorIntersectionFinder(const SingleInteriorIntersectionFinder& other);
+    SingleInteriorIntersectionFinder& operator=(const SingleInteriorIntersectionFinder& rhs);
 };
 
 } // namespace geos.noding
