@@ -1,10 +1,10 @@
-// $Id: DistanceOpTest.cpp 2276 2009-03-11 12:51:20Z mloskot $
+// $Id: DistanceOpTest.cpp 2554 2009-06-06 21:14:51Z strk $
 // 
 // Test Suite for geos::operation::distance::DistanceOp class.
 
-// TUT
-#include <tut.h>
-// GEOS
+// tut
+#include <tut.hpp>
+// geos
 #include <geos/operation/distance/DistanceOp.h>
 #include <geos/platform.h>
 #include <geos/geom/Coordinate.h>
@@ -13,7 +13,9 @@
 #include <geos/algorithm/PointLocator.h>
 #include <geos/io/WKTReader.h>
 #include <geos/geom/CoordinateSequence.h>
+// std
 #include <memory>
+#include <string>
 #include <vector>
 
 namespace tut
@@ -29,6 +31,7 @@ namespace tut
 		geos::io::WKTReader wktreader;
 
 		typedef geos::geom::Geometry::AutoPtr GeomPtr;
+		typedef std::auto_ptr<geos::geom::CoordinateSequence> CSPtr;
 
 		test_distanceop_data()
             : gf(), wktreader(&gf)
@@ -48,6 +51,7 @@ namespace tut
 	void object::test<1>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(0 0)");
 		std::string wkt1("POINT(10 0)");
@@ -58,10 +62,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
-        geos::geom::CoordinateSequence* coords = dist.closestPoints();
-        ensure(0 != coords);
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(0, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(10, 0));
 	}
 
 	template<>
@@ -69,6 +72,7 @@ namespace tut
 	void object::test<2>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(0 0)");
 		std::string wkt1("MULTIPOINT(10 0, 50 30)");
@@ -79,8 +83,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(0, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(10, 0));
 
 	}
 
@@ -89,6 +94,7 @@ namespace tut
 	void object::test<3>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(3 0)");
 		std::string wkt1("LINESTRING(0 10, 50 10, 100 50)");
@@ -99,8 +105,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(3, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(3, 10));
 
 	}
 
@@ -109,6 +116,7 @@ namespace tut
 	void object::test<4>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(3 0)");
 		std::string wkt1("MULTILINESTRING((34 54, 60 34),(0 10, 50 10, 100 50))");
@@ -119,8 +127,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(3, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(3, 10));
 
 	}
 
@@ -129,6 +138,7 @@ namespace tut
 	void object::test<5>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(35 60)");
 		std::string wkt1("POLYGON((34 54, 60 34, 60 54, 34 54),(50 50, 52 50, 52 48, 50 48, 50 50))");
@@ -139,8 +149,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 
 	}
 
@@ -149,6 +160,7 @@ namespace tut
 	void object::test<6>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(35 60)");
 		std::string wkt1("MULTIPOLYGON(((34 54, 60 34, 60 54, 34 54),(50 50, 52 50, 52 48, 50 48, 50 50)),( (100 100, 150 100, 150 150, 100 150, 100 100),(120 120, 120 130, 130 130, 130 120, 120 120)) ))");
@@ -159,8 +171,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 
 	}
 
@@ -169,6 +182,7 @@ namespace tut
 	void object::test<7>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(35 60)");
 
@@ -182,9 +196,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
-
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 	}
 
 	template<>
@@ -192,6 +206,7 @@ namespace tut
 	void object::test<8>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POINT(35 60)");
 
@@ -203,11 +218,9 @@ namespace tut
 
 		DistanceOp dist(g0.get(), g1.get());
 
-		ensure_equals(dist.distance(), DoubleInfinity);
-
-		// TODO: test closestPoints() and
-		//       closestLocations()
+		ensure_equals(dist.distance(), DoubleMax);
 
+		ensure_equals(dist.closestPoints(), (void*)0);
 	}
 
 	template<>
@@ -215,6 +228,7 @@ namespace tut
 	void object::test<9>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(10 0, 50 30)");
 		std::string wkt1("POINT(10 0)");
@@ -225,8 +239,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 0);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(10, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(10, 0));
 
 	}
 
@@ -235,6 +250,7 @@ namespace tut
 	void object::test<10>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(10 0, 50 30)");
 		std::string wkt1("MULTIPOINT(0 0, 150 30)");
@@ -245,8 +261,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(10, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(0, 0));
 
 	}
 
@@ -255,6 +272,7 @@ namespace tut
 	void object::test<11>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(3 0, 200 30)");
 		std::string wkt1("LINESTRING(0 10, 50 10, 100 50)");
@@ -265,8 +283,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(3, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(3, 10));
 
 	}
 
@@ -275,6 +294,7 @@ namespace tut
 	void object::test<12>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(3 0, -50 30)");
 		std::string wkt1("MULTILINESTRING((34 54, 60 34),(0 10, 50 10, 100 50))");
@@ -285,8 +305,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 10);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(3, 0));
+        	ensure_equals(cs->getAt(1), Coordinate(3, 10));
 
 	}
 
@@ -295,6 +316,7 @@ namespace tut
 	void object::test<13>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(-100 0, 35 60)");
 		std::string wkt1("POLYGON((34 54, 60 34, 60 54, 34 54),(50 50, 52 50, 52 48, 50 48, 50 50))");
@@ -305,8 +327,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 
 	}
 
@@ -315,6 +338,7 @@ namespace tut
 	void object::test<14>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(-100 0, 35 60)");
 		std::string wkt1("MULTIPOLYGON(((34 54, 60 34, 60 54, 34 54),(50 50, 52 50, 52 48, 50 48, 50 50)),( (100 100, 150 100, 150 150, 100 150, 100 100),(120 120, 120 130, 130 130, 130 120, 120 120)) ))");
@@ -325,9 +349,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
-
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 	}
 
 	template<>
@@ -335,6 +359,7 @@ namespace tut
 	void object::test<15>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(-100 0, 35 60)");
 
@@ -348,8 +373,9 @@ namespace tut
 
 		ensure_equals(dist.distance(), 6);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	CSPtr cs ( dist.closestPoints());
+        	ensure_equals(cs->getAt(0), Coordinate(35, 60));
+        	ensure_equals(cs->getAt(1), Coordinate(35, 54));
 
 	}
 
@@ -358,6 +384,7 @@ namespace tut
 	void object::test<16>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("MULTIPOINT(-100 0, 35 60)");
 
@@ -369,10 +396,9 @@ namespace tut
 
 		DistanceOp dist(g0.get(), g1.get());
 
-		ensure_equals(dist.distance(), DoubleInfinity);
+		ensure_equals(dist.distance(), DoubleMax);
 
-		// TODO: test closestPoints() and
-		//       closestLocations()
+        	ensure_equals(dist.closestPoints(), (void*)0);
 
 	}
 
@@ -383,6 +409,7 @@ namespace tut
 	void object::test<17>()
 	{
 		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
 
 		std::string wkt0("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))");
 		std::string wkt1("POLYGON((1.25 0.25, 1.25 0.75, 1.75 0.75, 1.75 0.25, 1.25 0.25))");
@@ -393,9 +420,39 @@ namespace tut
 		DistanceOp dist(g0.get(), g1.get());
 		ensure_equals(dist.distance(), 0.25);
 
-        // FIXME: This operation crashes becasue of both GeometryLocation's are NULL
-        //        in DistanceOp::closestPoints()
-        //geos::geom::CoordinateSequence* cs dist.closestPoints();
+        	CSPtr cs ( dist.closestPoints());
+		ensure_equals(cs->getAt(0), Coordinate(1, 0.25));
+		ensure_equals(cs->getAt(1), Coordinate(1.25, 0.25));
+	}
+
+	// Test for isWithinDistance
+	template<>
+	template<>
+	void object::test<18>()
+	{
+		using geos::operation::distance::DistanceOp;
+		using geos::geom::Coordinate;
+
+		std::string wkt0("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))");
+		std::string wkt1("POLYGON((1.25 0.25, 1.25 0.75, 1.75 0.75, 1.75 0.25, 1.25 0.25))");
+
+		GeomPtr g0(wktreader.read(wkt0));
+		GeomPtr g1(wktreader.read(wkt1));
+
+		ensure_equals(DistanceOp::distance(*g0, *g1), 0.25);
+		ensure(DistanceOp::isWithinDistance(*g0, *g1, 0.26));
+		ensure(!DistanceOp::isWithinDistance(*g0, *g1, 0.24));
+
+		wkt0 = "LINESTRING(0 0, 0 1, 1 1, 1 0, 0 0)";
+		wkt1 = "LINESTRING(2 0, 10 1, 10 10)";
+		g0.reset(wktreader.read(wkt0));
+		g1.reset(wktreader.read(wkt1));
+
+		ensure_equals(DistanceOp::distance(*g0, *g1), 1);
+		ensure(DistanceOp::isWithinDistance(*g0, *g1, 2));
+		ensure(!DistanceOp::isWithinDistance(*g0, *g1, 0.8));
+
+		// TODO: test closestPoints
 	}
 
 	// TODO: finish the tests by adding:
